function [ cc, si, azc1, azc2 ] = MG_Tools_CC_AZC(sessions, params)
% MG_TOOLS_CC_AZC - new CC algorithm based on (1) detect Acc-Zero-Cross for
% each player, search for <1,1>, (2) reject pair with large <dV, dT, dT-Peaks>, using fixed threshold
% [ cc si  azc1 azc2 ] = MG_Tools_CC_AZC(sessions, params) - 
% input:
% sessions - cell array, regular MG sessions
% params (optional) - 
% params.dV_thr =  0.95;
% params.dT_thr = 0.09;
% params.dT_Peaks_thr = 0.3; 
% => for CC, need to be less then all three
% params.Include_Anti_Phase = 0;
% 
% 20-05-14: add minimal CC demand for atleasat two segments (a full cycle).
% params.Remove_Single_CC_Segment (=1)
%
% 01-05-15: add dealing with 'missing' rounds (marked in .leaders)
%
% 30-08-16: add params1.butter_filter , to use 'butter' filter for pos => vel, to match Jason for Drift_Rate_1
% params1.butter_filter (defulat = 1) - put 0 to use bfilt instead of butter filter 
%
% output:
% cc - {sessions, rounds}, with 0/1 vectors for CC samples, same length as position vectors
% si - list of segments, with  [1) #session, 2) #round, 3) start-pos, 4) end-pos, 5) dv, 6) dt, 7) dt_peaks, 
% 8) acz1, 9) acz2, 10) 'trueCC'  = EMPTY, for matching to collect_cc_ncc_ 11) 'algCC' = EMPTY (same) 12) phase (1=same, 0=anti))]
% 13) NewCC (AZC=<1,1> & OK for <dv,dt,dt-peaks> thresholds)
%
% 21.12.14 - collect AZC 0/1 vector for <sessions, rounds> to allow later jitter-count
% azc1, azc2 - {sessions, rounds}, with 0/1 vecotrs for Acc-Zero-Cross
% (collected from MG_Tools_Segments_Acc_Zero_Cross)
%
% assume here for all we are in 50hz world (defined in MG_Default_Const)

% Default Values for Optional Inputs %
if ~exist('params','var') || isempty(params)
	params = [];
end

% Default Values for Optional Params Fields %
if ~isfield(params,'dV_thr') || isempty(params.dV_thr),
	params.dV_thr =  0.95;
end

if ~isfield(params,'dT_thr') || isempty(params.dT_thr),
	params.dT_thr = 0.09;
end

if ~isfield(params,'dT_Peaks_thr') || isempty(params.dT_Peaks_thr),
	params.dT_Peaks_thr = 0.3;
end

if ~isfield(params,'Include_Anti_Phase') || isempty(params.Include_Anti_Phase),
	params.Include_Anti_Phase = 0;
end

params = add_default( params, 'Remove_Single_CC_Segment',1);

params = add_default( params, 'butter_filter', 1);

% -------- go over all rounds -------------
si = [];
for i = 1 : length( sessions ),
    sprintf('game %d',i)
    if ~isfield( sessions{i}, 'trials' ) || isempty( sessions{i}.trials ), continue, end
    for j = 1 : length( sessions{i}.trials ),    
            %sprintf('session %d, round %d',i,j), % for display,  uncomment
            
            if strcmp( sessions{i}.leaders{j}, 'missing' ),
                azc1{i,j} = NaN;
                azc2{i,j} = NaN;
                tmp = [];
            else                           
                % collect segment information for this round
                x = sessions{i}.trials{j};
                [tmp, azc1{i,j}, azc2{i,j}] = do_one( x, params ); 
            end
            if ~isempty( tmp ),
                num_seg = size( tmp, 1 );
                si = cat(1, si, [ i * ones( num_seg, 1 ), j * ones( num_seg, 1 ), tmp ]);
            else
                cc = []; si = []; azc1 = []; azc2 = [];
                continue
            end
    end
end

% --------------- classity AZC<1,1> ------------------

% get parameters for CC
dV = si(:,5);
dT = si(:,6);
dTpeaks = si(:,7);
azcCC =  si(:,8) == 1 & si(:,9) == 1;
Phase = si(:,12);

% CC by AZC, <dv,dt,dt-peaks> Thresholds (and Phase if needed)
if params.Include_Anti_Phase,
    si(:,13) =  (azcCC == 1) & (dT < params.dT_thr) &  (dV < params.dV_thr) & (dTpeaks < params.dT_Peaks_thr);
else % Phase Only
    si(:,13) =  (azcCC == 1) & (dT < params.dT_thr) &  (dV < params.dV_thr) & (dTpeaks < params.dT_Peaks_thr) & Phase;
end


% check for too small, demand at least 100ms
ix = find( si(:,4) - si(:,3) > 4);
si = si(ix,:); %#ok<*FNDSB>

% remove single CC segment
if params.Remove_Single_CC_Segment,
    
        % - add real no-motion segments and mark them
        % - add one dummy-segment at start/end trial, with 0 CC, to allow cleaning of a singel segment at start/end of trials
        % - add #14 coloum with 1 for real segment, 0 no motion / dummy-segment
        si2 = add_no_motion( si ); 
        si2(:,13) =  bwmorph(  si2(:,13), 'clean' );
        ix = find( si2(:,14) );
        si = si2(ix,1:13);
end

% ------------- set CC vectors
for i = 1 : length( sessions ),
    if ~isfield( sessions{i}, 'trials' ) || isempty( sessions{i}.trials ), continue, end
    for j = 1 : length( sessions{i}.trials ),
        if strcmp( sessions{i}.leaders{j}, 'missing' ),
            cc{i,j} = NaN;
        else
            cc{i,j} = zeros( size( sessions{i}.trials{j}, 1 ), 1 ); %#ok<*AGROW>
            ix = find( si(:,1) == i & si(:,2) == j & si(:,13) == 1);
            for k = 1 : length(ix),
                from = si(ix(k), 3);
                to = si(ix(k), 4);
                cc{i,j}( from : to ) = 1;
            end
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%
function [seg, azc1, azc2] = do_one( x, params1 )
% x - [t (sec), p1 (position, in ticks), p2]
% params1 - allow to enter 'params1.butter_filter = 1' for Drift_Rate_15
%
% outout:
% seg - [1) start pos, 2) end pos, 3) dv, 4) dt, 5) dt_peaks, 6) acz1, 7) acz2, 8) NaN 9) NaN 10) phase ]
% azc1, azc2 - acc-zero-cross vectors for the two players (same size as <x>, 1 == AZC point)

if isempty( x ), 
    seg = [];
    azc1 = zeros( size(x,1), 1 ); 
    azc2 = zeros( size(x,1), 1 ); 
    return;
end

% create demi-session structure
sess{1}.session_data = [];
sess{1}.leaders{1} = 'Both';
sess{1}.session_index = 1;
sess{1}.trials{1} =  x;

% get segments featues, over all vector
params1.no_progress_bar = 1;
params1.min_seg_time = 0.1; % allow up to 2.5hz motions

[~, xdv, ~, xdt, ~, xdt_peaks, ~, xphase] = MG_Round_dV_from_Segments( sess, params1 );

% get segments & Acc-Zero-Cross points
df = MG_Default_Const;
[ seg, azc_num, azc1, azc2] = MG_Tools_Segments_Acc_Zero_Cross( x, df.Samples_Per_Sec, params1 );

if isempty( seg ), return, end

% collect segments data
for i = 1 : size(seg,1),
    a = seg(i,1);
    b = seg(i,2);
    
    % [start pos, end pos, dv, dt, dt_peaks, acz1, acz2]
    seg(i,3:5) = [ nanmean(xdv{1}(a:b)), nanmean(xdt{1}(a:b)), nanmean(xdt_peaks{1}(a:b)) ];
    seg(i,6:7) = azc_num(i,:);
    
    % 05-05-14: use MODE for Phase, as segment here does not match exact
    % segments (padding?),  and want to force to 0/1
    tmp = xphase{1}(a:b);
    seg(i,10) = mode( tmp(~isnan(tmp)) );    
    if isnan(  seg(i,10) ), seg(i,10) = -1; end
end

%%%%%%%%%%%%%%%%%%%
function si2 = add_no_motion( si )

if isempty(si), si2 = si; return; end

allowed_distance_ratio = 0.2; % allowed distance, in ratio of segment length, of between-segment-distance

dummy_seg   = NaN * zeros(1,14);
dummy_seg(13) = 0; % no CC here
dummy_seg(14) = 0; % 'no-moion'/dummy segment

% start first round with dummy before .. 
si2(1,:) = dummy_seg;
si2(1,1:2) = si(1,1:2); % same <session, round>
si2(1,3:4) = [-10, si(1,3) - 1]; 

% .. then real first segment
si2(2,:) = [si(1,:), 1];
si2(2,14) = 1;
si2_ind = 3;
round_ind = si(1,2);

for i = 2 : size(si,1),
    
    % add dummy-segment at the begining of a new round
    if si(i,2) ~= round_ind,
        round_ind = si(i,2);
        % start new round with dummy 
        si2(si2_ind,:) = dummy_seg;
        si2(si2_ind,1:2) = si(i,1:2); % same <session, round> as current new round
        si2(si2_ind,3:4) = [-10, si(i,3) - 1]; 
        si2_ind = si2_ind + 1;        
    else
        % if not first segment in round, check for 'no-motion', add if needed
        dist_in_samples = si(i,3) - si(i-1,4);
        dist_in_ratio_of_first_seg = dist_in_samples / ( si(i-1,4) - si(i-1,3) );
                 
        if dist_in_ratio_of_first_seg > allowed_distance_ratio,
            % add no-motion segment
            si2(si2_ind,:)  = dummy_seg;
            si2(si2_ind,1:2) = si(i,1:2); % same <session, round> as current
            si2(si2_ind,3:4) = [si(i-1,4) + 1, si(i,3) - 1]; % no-motion segment from end-of-previous to start-of-current
            si2_ind = si2_ind + 1;
        end
    end
    
   % after all possible dummy additions, add this real segment    
    si2(si2_ind,:) = [ si(i,:), 1 ];  
    si2_ind = si2_ind + 1;    
end   

% end last round with dummy
si2(si2_ind,:) = dummy_seg;
si2(si2_ind,1:2) = si(i,1:2); % same <session, round> as last
si2(si2_ind,3:4) = [si(i,4) + 1, si(i,4) + 10];     