function azc =  find_acc_zero_cross(p, fps, params)
%  FIND_ACC_ZERO_CROSS - from Jason (Meyer / Dounskai method)
% azc =  find_acc_zero_cross(p, fps, params) -
% input:
% p - position vercor
% fps - frame-per-sec (Hz)
% params (optional) -
%  .diff_method = 'butter' (defualt) or 'bfilt'
% . plot (=0)
% NEW 23-Dec-14, remove AZC in no-position-change points (enter due to Vel filtering)
% .clean_zero_vel (=1) - make sure to remove AZC in no-motion segments
%
% .clean_constant_vel (=0) - deal with sub-segment which are very close to
% zero-acceltation over a period, get only one (not two) AZC from these.
% currently use hard threshold <MaxVel(round)/75. Can improve by taking
% ratio only w.r.t. current seg veocities.
% 
% 08.09.2016:
% params.butter_filter - (=1). will effect .diff_method, to match other functions. will only have effect if exist and == 0.
%
% output:
% azc - vecotr (length of <p>) with 1 for places of accelartion-zero-crossing
%

% Default Values for Optional Inputs %
if ~exist('params','var') || isempty(params)
    params = [];
end

% Default Values for Optional Params Fields %
params = add_default( params, 'diff_method', 'butter' );
params = add_default( params, 'plot', 0 );
params = add_default( params, 'clean_zero_vel', 1 );

params = add_default( params, 'clean_constant_vel', 0 );

if isfield( params, 'butter_filter'),
    if params.butter_filter == 0,
             params.diff_method = 'bfilt';
    else
           params.diff_method = 'butter';
    end 
end

params.do_acc = 1;
switch params.diff_method,
    case 'butter',
        % BUTTER code now: filter position=> Diff => filter Vel=>Diff=>Filter Acc, return
        params.cutoff = 5; % too low?
        p2 = remove_nan( p );        % butter filter does not like Nan
        [vel, acc]  = vel_acc_with_butter(p2, fps,  params);
    case 'bfilt',
        % BFILT code now: filter position=> Diff => filter Vel=>Diff=> return (no smooth at last stage)
        [vel, acc] = vel_acc_with_bfilt(p, params );
    otherwise,
        error('unknown diff_method')
end

if size(p,1) < size(p,2),
    accshift1 = [acc(2:end), NaN];
else
    accshift1 = [acc(2:end); NaN];
end

azc = abs(sign(acc)-sign(accshift1))>0;

% this is perviously from Jason
%small_acc_error = max(acc)  * 10^-10;
%azc = abs(sign(acc)-sign(accshift1))>0 & abs(acc-accshift1) < small_acc_error;


if params.clean_zero_vel,
    dp = [0; diff(p)];
    azc( dp == 0 ) = 0;
end

if params.clean_constant_vel,    
    vel_thr = max(abs(vel)) / 75;
    zi = find( azc );
    for i = 1 : length( zi ) - 1,
        vel_seg = vel(zi(i) : zi(i+1));
        if abs( max(vel_seg) - min( vel_seg ) ) < vel_thr,
            azc(zi(i)) = 0;
            i = i + 1; %#ok<*FXSET>
        end
    end    
end



if params.plot,
    
    subplot 131;
    plot(p);
    ylabel('Position')
    
    subplot 132;
    if ~strcmp( params.diff_method, 'butter' ),
        vel = vel * 50; % => mm/sec
    end
    plot(vel);
    hold on
    zc1 = find( azc );
    plot( zc1, vel(zc1),'.g','MarkerSize',6 )
    ax = axis;
    plot( [ax(1),ax(2)], [0 0], 'k' )
    hold off
    ylabel('Velocity +AZC')
    
    
    subplot 133;
    plot(acc);
    hold on
    plot( [ax(1),ax(2)], [0 0], 'k' )
    hold off
    ylabel('Acceleration')
    
    for i=1:3,
        subplot(1,3,i);
        ax(i)=gca;
    end
    linkaxes(ax,'x')
end


